#ifndef LIB_DBG_DEBUG_GLOBAL_EVENT_HANDLER_TRACER_MQH_INCLUDED
#define LIB_DBG_DEBUG_GLOBAL_EVENT_HANDLER_TRACER_MQH_INCLUDED
#property version "5.50"
/**********************************************************************************
 * Copyright (C) 2020 Dominik Egert <info@freie-netze.de>
 *
 * This file is the debugger main user macros include file.
 *
 * Lisence applied: GPLv2
 *                  https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 *
 * Author Dominik Egert / Freie Netze UG.
 **********************************************************************************
 *
 *  Version: 4120
 *  State: public
 *
 *  File information
 *  ================
 *
*/




//*********************************************************************************************************************************************************/
//  BEGIN Debugging trace event handlers
//


/////////////////////////////////////////////////////////////////////////////////////////////////////
//
// MQL5 Event handler trace 
//
#ifndef __MQL4__

//#define LIB_DEBUG_TRACE_EVENT_HANDLER_ONSTART
//#define LIB_DEBUG_TRACE_EVENT_HANDLER_ONINIT
//#define LIB_DEBUG_TRACE_EVENT_HANDLER_ONDEINIT
//#define LIB_DEBUG_TRACE_EVENT_HANDLER_ONCALCULATE_OHLC
//#define LIB_DEBUG_TRACE_EVENT_HANDLER_ONCALCULATE_PRICE
//#define LIB_DEBUG_TRACE_EVENT_HANDLER_ONTIMER
//#define LIB_DEBUG_TRACE_EVENT_HANDLER_ONTRADE
//#define LIB_DEBUG_TRACE_EVENT_HANDLER_ONTRADETRANSACTION
//#define LIB_DEBUG_TRACE_EVENT_HANDLER_ONBOOKEVENT
//#define LIB_DEBUG_TRACE_EVENT_HANDLER_ONCHARTEVENT
//#define LIB_DEBUG_TRACE_EVENT_HANDLER_ONTESTER
//#define LIB_DEBUG_TRACE_EVENT_HANDLER_ONTESTERINIT
//#define LIB_DEBUG_TRACE_EVENT_HANDLER_ONTESTERDEINIT
//#define LIB_DEBUG_TRACE_EVENT_HANDLER_ONTESTERPASS

    #ifdef LIB_DEBUG_TRACE_HEAP_MEMORY
        #ifdef new
            #undef new

        #endif
        #ifdef delete 
            #undef delete
            
        #endif

    #endif
    #ifndef __MQL4__
        namespace dbg_lib
        {

    #endif

        typedef void (*VOID_FPTR)();
        typedef int (*INT_FPTR)();

        template <typename T>
        struct __auto_destroy_ptr
        {
            T* _ptr;
            __auto_destroy_ptr() :
                _ptr(NULL)
            { };
        
            __auto_destroy_ptr(T* p_in) :
                _ptr(p_in)
            { };
        
            ~__auto_destroy_ptr()
            {
                if( (_ptr != NULL)
                 && (CheckPointer(_ptr) != POINTER_INVALID) )
                { delete _ptr; }
            };
        
            void operator=(T* p_in)
            { _ptr = p_in; };
        };
        
        class __event_handler_fptr
        {
            public:
            VOID_FPTR   v_func;
            INT_FPTR    i_func;
            
            __event_handler_fptr(VOID_FPTR p_in) :
                v_func(p_in),
                i_func(NULL)
            { };
        
            __event_handler_fptr(INT_FPTR p_in) :
                v_func(NULL),
                i_func(p_in)
            { };
        
            int call_handler()
            {
                if(v_func != NULL)
                {
                    v_func();
                    return(NULL);
                }
                return(i_func());
            };
            
            const bool is_void_fptr()
            { return(v_func != NULL); }
        };
        __event_handler_fptr* start_ptr = NULL;
        __auto_destroy_ptr<__event_handler_fptr> auto_destroy_start_object;
        __event_handler_fptr* init_ptr = NULL;
        __auto_destroy_ptr<__event_handler_fptr> auto_destroy_init_object;
        __event_handler_fptr* tester_ptr = NULL;
        __auto_destroy_ptr<__event_handler_fptr> auto_destroy_tester_object;

    #ifndef __MQL4__
        }; // Namespace: dbg_lib::

    #endif


    #ifdef LIB_DEBUG_TRACE_EVENT_HANDLER_ONSTART
        int OnStart()
        {
            if(LIB_DBG_NAMESPACE(dbg_lib, start_ptr) == NULL)
            {
                LIB_DBG_NAMESPACE(dbg_lib, start_ptr) = new LIB_DBG_NAMESPACE(dbg_lib, __event_handler_fptr)(dbg_OnStart);
                LIB_DBG_NAMESPACE(dbg_lib, auto_destroy_start_object) = LIB_DBG_NAMESPACE(dbg_lib, start_ptr);
            }
            static ulong call_counter = NULL;
            LIB_DBG_NAMESPACE(dbg_lib, dbg_trace_begin)(__dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, call_counter);
            const int OnStart_return = LIB_DBG_NAMESPACE(dbg_lib, start_ptr).call_handler();
            if(!LIB_DBG_NAMESPACE(dbg_lib, start_ptr).is_void_fptr())
            { LIB_DBG_NAMESPACE(dbg_lib, dbg_trace_end)(__dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, LIB_DBG_NAMESPACE(dbg_lib, var_out)(__DBG_STRINGIFY_MACRO((OnStart_return)), OnStart_return)); }
            else
            { LIB_DBG_NAMESPACE(dbg_lib, dbg_trace_end)(__dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, NULL); }
            return(OnStart_return);
        };
        #define OnStart                 dbg_OnStart
    
    #endif
    #ifdef LIB_DEBUG_TRACE_EVENT_HANDLER_ONINIT
        int OnInit()
        {
            if(LIB_DBG_NAMESPACE(dbg_lib, init_ptr) == NULL)
            {
                LIB_DBG_NAMESPACE(dbg_lib, init_ptr) = new LIB_DBG_NAMESPACE(dbg_lib, __event_handler_fptr)(dbg_OnInit);
                LIB_DBG_NAMESPACE(dbg_lib, auto_destroy_init_object) = LIB_DBG_NAMESPACE(dbg_lib, init_ptr);
            }
            ulong call_counter = NULL;
            LIB_DBG_NAMESPACE(dbg_lib, dbg_trace_begin)(__dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, call_counter);
            const ulong start = GetMicrosecondCount();
            const int OnInit_return = LIB_DBG_NAMESPACE(dbg_lib, init_ptr).call_handler();
            const ulong OnInit_runtime = GetMicrosecondCount() - start;
            LIB_DBG_NAMESPACE(dbg_lib, dbg_printf)(LIB_DBG_NAMESPACE(dbg_lib, dbg_context_msg)(DBG_OUTPUT_PREFIX, __dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, LIB_DBG_NAMESPACE(dbg_lib, var_out)(__DBG_STRINGIFY_MACRO((OnInit_runtime)), (OnInit_runtime)), "=", DBG_MSG_FORMAT_RIGHT_COLUMN_SPACER));
            if(!LIB_DBG_NAMESPACE(dbg_lib, init_ptr).is_void_fptr())
            { LIB_DBG_NAMESPACE(dbg_lib, dbg_trace_end)(__dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, LIB_DBG_NAMESPACE(dbg_lib, var_out)(__DBG_STRINGIFY_MACRO((OnInit_return)), OnInit_return)); }
            else
            { LIB_DBG_NAMESPACE(dbg_lib, dbg_trace_end)(__dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, NULL); }
            return(OnInit_return);
        };
        #define OnInit                  dbg_OnInit
    
    #endif
    #ifdef LIB_DEBUG_TRACE_EVENT_HANDLER_ONDEINIT
        void OnDeinit(const int reason)
        {
            ulong call_counter = NULL;
            LIB_DBG_NAMESPACE(dbg_lib, dbg_trace_begin)(__dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, call_counter);
            dbg_OnDeinit(reason);
            LIB_DBG_NAMESPACE(dbg_lib, dbg_trace_end)(__dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, NULL);
            return;
        };
        #define OnDeinit                dbg_OnDeinit
    
    #endif
    #ifdef LIB_DEBUG_TRACE_EVENT_HANDLER_ONTICK
        void OnTick()
        {
            ulong call_counter = NULL;
            LIB_DBG_NAMESPACE(dbg_lib, dbg_trace_begin)(__dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, call_counter);
            const ulong start = GetMicrosecondCount();
            dbg_OnTick();
            const ulong OnTick_runtime = GetMicrosecondCount() - start;
            LIB_DBG_NAMESPACE(dbg_lib, dbg_printf)(LIB_DBG_NAMESPACE(dbg_lib, dbg_context_msg)(DBG_OUTPUT_PREFIX, __dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, LIB_DBG_NAMESPACE(dbg_lib, var_out)(__DBG_STRINGIFY_MACRO((OnTick_runtime)), (OnTick_runtime)), "=", DBG_MSG_FORMAT_RIGHT_COLUMN_SPACER));
            LIB_DBG_NAMESPACE(dbg_lib, dbg_trace_end)(__dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, NULL);
            return;
        };
        #define OnTick                  dbg_OnTick
    
    #endif
    #ifdef LIB_DEBUG_TRACE_EVENT_HANDLER_ONCALCULATE_OHLC
        int OnCalculate(const int rates_total,
                        const int prev_calculated,
                        const datetime& time[],
                        const double& open[],
                        const double& high[],
                        const double& low[],
                        const double& close[],
                        const long& tick_volume[],
                        const long& volume[],
                        const int& spread[])
        {
            LIB_DBG_NAMESPACE(dbg_lib, dbg_printf)(LIB_DBG_NAMESPACE(dbg_lib, dbg_context_msg)(DBG_OUTPUT_PREFIX, __dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, LIB_DBG_NAMESPACE(dbg_lib, var_out)(__DBG_STRINGIFY_MACRO((rates_total)),                      (rates_total)),                     "=", DBG_MSG_FORMAT_RIGHT_COLUMN_SPACER));
            LIB_DBG_NAMESPACE(dbg_lib, dbg_printf)(LIB_DBG_NAMESPACE(dbg_lib, dbg_context_msg)(DBG_OUTPUT_PREFIX, __dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, LIB_DBG_NAMESPACE(dbg_lib, var_out)(__DBG_STRINGIFY_MACRO((prev_calculated)),                  (prev_calculated)),                 "=", DBG_MSG_FORMAT_RIGHT_COLUMN_SPACER));
            LIB_DBG_NAMESPACE(dbg_lib, dbg_printf)(LIB_DBG_NAMESPACE(dbg_lib, dbg_context_msg)(DBG_OUTPUT_PREFIX, __dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, LIB_DBG_NAMESPACE(dbg_lib, var_out)(__DBG_STRINGIFY_MACRO((time[0])),                          (time[0])),                         "=", DBG_MSG_FORMAT_RIGHT_COLUMN_SPACER));
            LIB_DBG_NAMESPACE(dbg_lib, dbg_printf)(LIB_DBG_NAMESPACE(dbg_lib, dbg_context_msg)(DBG_OUTPUT_PREFIX, __dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, LIB_DBG_NAMESPACE(dbg_lib, var_out)(__DBG_STRINGIFY_MACRO((time[rates_total - 1])),            (time[rates_total - 1])),           "=", DBG_MSG_FORMAT_RIGHT_COLUMN_SPACER));
            LIB_DBG_NAMESPACE(dbg_lib, dbg_printf)(LIB_DBG_NAMESPACE(dbg_lib, dbg_context_msg)(DBG_OUTPUT_PREFIX, __dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, LIB_DBG_NAMESPACE(dbg_lib, var_out)(__DBG_STRINGIFY_MACRO((open[0])),                          (open[0])),                         "=", DBG_MSG_FORMAT_RIGHT_COLUMN_SPACER));
            LIB_DBG_NAMESPACE(dbg_lib, dbg_printf)(LIB_DBG_NAMESPACE(dbg_lib, dbg_context_msg)(DBG_OUTPUT_PREFIX, __dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, LIB_DBG_NAMESPACE(dbg_lib, var_out)(__DBG_STRINGIFY_MACRO((open[rates_total - 1])),            (open[rates_total - 1])),           "=", DBG_MSG_FORMAT_RIGHT_COLUMN_SPACER));
            LIB_DBG_NAMESPACE(dbg_lib, dbg_printf)(LIB_DBG_NAMESPACE(dbg_lib, dbg_context_msg)(DBG_OUTPUT_PREFIX, __dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, LIB_DBG_NAMESPACE(dbg_lib, var_out)(__DBG_STRINGIFY_MACRO((high[0])),                          (high[0])),                         "=", DBG_MSG_FORMAT_RIGHT_COLUMN_SPACER));
            LIB_DBG_NAMESPACE(dbg_lib, dbg_printf)(LIB_DBG_NAMESPACE(dbg_lib, dbg_context_msg)(DBG_OUTPUT_PREFIX, __dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, LIB_DBG_NAMESPACE(dbg_lib, var_out)(__DBG_STRINGIFY_MACRO((high[rates_total - 1])),            (high[rates_total - 1])),           "=", DBG_MSG_FORMAT_RIGHT_COLUMN_SPACER));
            LIB_DBG_NAMESPACE(dbg_lib, dbg_printf)(LIB_DBG_NAMESPACE(dbg_lib, dbg_context_msg)(DBG_OUTPUT_PREFIX, __dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, LIB_DBG_NAMESPACE(dbg_lib, var_out)(__DBG_STRINGIFY_MACRO((low[0])),                           (low[0])),                          "=", DBG_MSG_FORMAT_RIGHT_COLUMN_SPACER));
            LIB_DBG_NAMESPACE(dbg_lib, dbg_printf)(LIB_DBG_NAMESPACE(dbg_lib, dbg_context_msg)(DBG_OUTPUT_PREFIX, __dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, LIB_DBG_NAMESPACE(dbg_lib, var_out)(__DBG_STRINGIFY_MACRO((low[rates_total - 1])),             (low[rates_total - 1])),            "=", DBG_MSG_FORMAT_RIGHT_COLUMN_SPACER));
            LIB_DBG_NAMESPACE(dbg_lib, dbg_printf)(LIB_DBG_NAMESPACE(dbg_lib, dbg_context_msg)(DBG_OUTPUT_PREFIX, __dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, LIB_DBG_NAMESPACE(dbg_lib, var_out)(__DBG_STRINGIFY_MACRO((close[0])),                         (close[0])),                        "=", DBG_MSG_FORMAT_RIGHT_COLUMN_SPACER));
            LIB_DBG_NAMESPACE(dbg_lib, dbg_printf)(LIB_DBG_NAMESPACE(dbg_lib, dbg_context_msg)(DBG_OUTPUT_PREFIX, __dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, LIB_DBG_NAMESPACE(dbg_lib, var_out)(__DBG_STRINGIFY_MACRO((close[rates_total - 1])),           (close[rates_total - 1])),          "=", DBG_MSG_FORMAT_RIGHT_COLUMN_SPACER));
            LIB_DBG_NAMESPACE(dbg_lib, dbg_printf)(LIB_DBG_NAMESPACE(dbg_lib, dbg_context_msg)(DBG_OUTPUT_PREFIX, __dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, LIB_DBG_NAMESPACE(dbg_lib, var_out)(__DBG_STRINGIFY_MACRO((tick_volume[0])),                   (tick_volume[0])),                  "=", DBG_MSG_FORMAT_RIGHT_COLUMN_SPACER));
            LIB_DBG_NAMESPACE(dbg_lib, dbg_printf)(LIB_DBG_NAMESPACE(dbg_lib, dbg_context_msg)(DBG_OUTPUT_PREFIX, __dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, LIB_DBG_NAMESPACE(dbg_lib, var_out)(__DBG_STRINGIFY_MACRO((tick_volume[rates_total - 1])),     (tick_volume[rates_total - 1])),    "=", DBG_MSG_FORMAT_RIGHT_COLUMN_SPACER));
            LIB_DBG_NAMESPACE(dbg_lib, dbg_printf)(LIB_DBG_NAMESPACE(dbg_lib, dbg_context_msg)(DBG_OUTPUT_PREFIX, __dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, LIB_DBG_NAMESPACE(dbg_lib, var_out)(__DBG_STRINGIFY_MACRO((volume[0])),                        (volume[0])),                       "=", DBG_MSG_FORMAT_RIGHT_COLUMN_SPACER));
            LIB_DBG_NAMESPACE(dbg_lib, dbg_printf)(LIB_DBG_NAMESPACE(dbg_lib, dbg_context_msg)(DBG_OUTPUT_PREFIX, __dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, LIB_DBG_NAMESPACE(dbg_lib, var_out)(__DBG_STRINGIFY_MACRO((volume[rates_total - 1])),          (volume[rates_total - 1])),         "=", DBG_MSG_FORMAT_RIGHT_COLUMN_SPACER));
            LIB_DBG_NAMESPACE(dbg_lib, dbg_printf)(LIB_DBG_NAMESPACE(dbg_lib, dbg_context_msg)(DBG_OUTPUT_PREFIX, __dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, LIB_DBG_NAMESPACE(dbg_lib, var_out)(__DBG_STRINGIFY_MACRO((spread[0])),                        (spread[0])),                       "=", DBG_MSG_FORMAT_RIGHT_COLUMN_SPACER));
            LIB_DBG_NAMESPACE(dbg_lib, dbg_printf)(LIB_DBG_NAMESPACE(dbg_lib, dbg_context_msg)(DBG_OUTPUT_PREFIX, __dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, LIB_DBG_NAMESPACE(dbg_lib, var_out)(__DBG_STRINGIFY_MACRO((soread[rates_total - 1])),          (spread[rates_total - 1])),         "=", DBG_MSG_FORMAT_RIGHT_COLUMN_SPACER));

            ulong call_counter = NULL;
            LIB_DBG_NAMESPACE(dbg_lib, dbg_trace_begin)(__dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, call_counter);
            const ulong start = GetMicrosecondCount();
            const int OnCalculate_return = dbg_OnCalculate(rates_total, prev_calculated, time, open, high, low, close, tick_volume, volume, spread);
            const ulong OnCalculate_runtime = GetMicrosecondCount() - start;
            LIB_DBG_NAMESPACE(dbg_lib, dbg_printf)(LIB_DBG_NAMESPACE(dbg_lib, dbg_context_msg)(DBG_OUTPUT_PREFIX, __dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, LIB_DBG_NAMESPACE(dbg_lib, var_out)(__DBG_STRINGIFY_MACRO((OnCalculate_runtime)), (OnCalculate_runtime)), "=", DBG_MSG_FORMAT_RIGHT_COLUMN_SPACER));
            LIB_DBG_NAMESPACE(dbg_lib, dbg_trace_end)(__dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, LIB_DBG_NAMESPACE(dbg_lib, var_out)(__DBG_STRINGIFY_MACRO((OnCalculate_return)), OnCalculate_return));
            return(OnCalculate_return);
        };
        #ifdef LIB_DEBUG_TRACE_EVENT_HANDLER_ONCALCULATE_PRICE
            #undef LIB_DEBUG_TRACE_EVENT_HANDLER_ONCALCULATE_PRICE
            
        #endif
        #define OnCalculate             dbg_OnCalculate

    #endif
    #ifdef LIB_DEBUG_TRACE_EVENT_HANDLER_ONCALCULATE_PRICE
        int OnCalculate(const int rates_total,
                        const int prev_calculated,
                        const int begin,
                        const double& price[])
        {
            LIB_DBG_NAMESPACE(dbg_lib, dbg_printf)(LIB_DBG_NAMESPACE(dbg_lib, dbg_context_msg)(DBG_OUTPUT_PREFIX, __dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, LIB_DBG_NAMESPACE(dbg_lib, var_out)(__DBG_STRINGIFY_MACRO((rates_total)),                      (rates_total)),                     "=", DBG_MSG_FORMAT_RIGHT_COLUMN_SPACER));
            LIB_DBG_NAMESPACE(dbg_lib, dbg_printf)(LIB_DBG_NAMESPACE(dbg_lib, dbg_context_msg)(DBG_OUTPUT_PREFIX, __dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, LIB_DBG_NAMESPACE(dbg_lib, var_out)(__DBG_STRINGIFY_MACRO((prev_calculated)),                  (prev_calculated)),                 "=", DBG_MSG_FORMAT_RIGHT_COLUMN_SPACER));
            LIB_DBG_NAMESPACE(dbg_lib, dbg_printf)(LIB_DBG_NAMESPACE(dbg_lib, dbg_context_msg)(DBG_OUTPUT_PREFIX, __dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, LIB_DBG_NAMESPACE(dbg_lib, var_out)(__DBG_STRINGIFY_MACRO((begin)),                            (begin)),                           "=", DBG_MSG_FORMAT_RIGHT_COLUMN_SPACER));
            LIB_DBG_NAMESPACE(dbg_lib, dbg_printf)(LIB_DBG_NAMESPACE(dbg_lib, dbg_context_msg)(DBG_OUTPUT_PREFIX, __dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, LIB_DBG_NAMESPACE(dbg_lib, var_out)(__DBG_STRINGIFY_MACRO((price[0])),                         (price[0])),                        "=", DBG_MSG_FORMAT_RIGHT_COLUMN_SPACER));
            LIB_DBG_NAMESPACE(dbg_lib, dbg_printf)(LIB_DBG_NAMESPACE(dbg_lib, dbg_context_msg)(DBG_OUTPUT_PREFIX, __dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, LIB_DBG_NAMESPACE(dbg_lib, var_out)(__DBG_STRINGIFY_MACRO((price[rates_total - 1])),           (price[rates_total - 1])),          "=", DBG_MSG_FORMAT_RIGHT_COLUMN_SPACER));

            ulong call_counter = NULL;
            LIB_DBG_NAMESPACE(dbg_lib, dbg_trace_begin)(__dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, call_counter);
            const ulong start = GetMicrosecondCount();
            const int OnCalculate_return = dbg_OnCalculate(rates_total, prev_calculated, time, open, high, low, close, tick_volume, volume, spread);
            const ulong OnCalculate_runtime = GetMicrosecondCount() - start;
            LIB_DBG_NAMESPACE(dbg_lib, dbg_printf)(LIB_DBG_NAMESPACE(dbg_lib, dbg_context_msg)(DBG_OUTPUT_PREFIX, __dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, LIB_DBG_NAMESPACE(dbg_lib, var_out)(__DBG_STRINGIFY_MACRO((OnCalculate_runtime)), (OnCalculate_runtime)), "=", DBG_MSG_FORMAT_RIGHT_COLUMN_SPACER));
            LIB_DBG_NAMESPACE(dbg_lib, dbg_trace_end)(__dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, LIB_DBG_NAMESPACE(dbg_lib, var_out)(__DBG_STRINGIFY_MACRO((OnCalculate_return)), OnCalculate_return));
            return(OnCalculate_return);
        };
        #define OnCalculate             dbg_OnCalculate
    
    #endif
    #ifdef LIB_DEBUG_TRACE_EVENT_HANDLER_ONTIMER
        void OnTimer()
        {
            ulong call_counter = NULL;
            LIB_DBG_NAMESPACE(dbg_lib, dbg_trace_begin)(__dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, call_counter);
            const ulong start = GetMicrosecondCount();
            dbg_OnTimer();
            const ulong OnTimer_runtime = GetMicrosecondCount() - start;
            LIB_DBG_NAMESPACE(dbg_lib, dbg_printf)(LIB_DBG_NAMESPACE(dbg_lib, dbg_context_msg)(DBG_OUTPUT_PREFIX, __dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, LIB_DBG_NAMESPACE(dbg_lib, var_out)(__DBG_STRINGIFY_MACRO((OnTimer_runtime)), (OnTimer_runtime)), "=", DBG_MSG_FORMAT_RIGHT_COLUMN_SPACER));
            LIB_DBG_NAMESPACE(dbg_lib, dbg_trace_end)(__dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, NULL);
            return;
        };
        #define OnTimer                 dbg_OnTimer
    
    #endif
    #ifdef LIB_DEBUG_TRACE_EVENT_HANDLER_ONTRADE
        void OnTrade()
        {
            ulong call_counter = NULL;
            LIB_DBG_NAMESPACE(dbg_lib, dbg_trace_begin)(__dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, call_counter);
            const ulong start = GetMicrosecondCount();
            dbg_OnTrade();
            const ulong OnTrade_runtime = GetMicrosecondCount() - start;
            LIB_DBG_NAMESPACE(dbg_lib, dbg_printf)(LIB_DBG_NAMESPACE(dbg_lib, dbg_context_msg)(DBG_OUTPUT_PREFIX, __dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, LIB_DBG_NAMESPACE(dbg_lib, var_out)(__DBG_STRINGIFY_MACRO((OnTrade_runtime)), (OnTrade_runtime)), "=", DBG_MSG_FORMAT_RIGHT_COLUMN_SPACER));
            LIB_DBG_NAMESPACE(dbg_lib, dbg_trace_end)(__dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, NULL);
            return;
        };
        #define OnTrade                 dbg_OnTrade
    
    #endif
    #ifdef LIB_DEBUG_TRACE_EVENT_HANDLER_ONTRADETRANSACTION
        void OnTradeTransaction(const MqlTradeTransaction& trans,
                                const MqlTradeRequest& request,
                                const MqlTradeResult& result)
        {
            LIB_DBG_NAMESPACE(dbg_lib, dbg_printf)(LIB_DBG_NAMESPACE(dbg_lib, dbg_context_msg)(DBG_OUTPUT_PREFIX, __dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, LIB_DBG_NAMESPACE(dbg_lib, var_out)(__DBG_STRINGIFY_MACRO((trans)),    (trans)),   "=", DBG_MSG_FORMAT_RIGHT_COLUMN_SPACER));
            LIB_DBG_NAMESPACE(dbg_lib, dbg_printf)(LIB_DBG_NAMESPACE(dbg_lib, dbg_context_msg)(DBG_OUTPUT_PREFIX, __dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, LIB_DBG_NAMESPACE(dbg_lib, var_out)(__DBG_STRINGIFY_MACRO((request)),  (request)), "=", DBG_MSG_FORMAT_RIGHT_COLUMN_SPACER));
            LIB_DBG_NAMESPACE(dbg_lib, dbg_printf)(LIB_DBG_NAMESPACE(dbg_lib, dbg_context_msg)(DBG_OUTPUT_PREFIX, __dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, LIB_DBG_NAMESPACE(dbg_lib, var_out)(__DBG_STRINGIFY_MACRO((result)),   (result)),  "=", DBG_MSG_FORMAT_RIGHT_COLUMN_SPACER));
            ulong call_counter = NULL;
            LIB_DBG_NAMESPACE(dbg_lib, dbg_trace_begin)(__dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, call_counter);
            const ulong start = GetMicrosecondCount();
            dbg_OnTradeTransaction();
            const ulong OnTradeTransaction_runtime = GetMicrosecondCount() - start;
            LIB_DBG_NAMESPACE(dbg_lib, dbg_printf)(LIB_DBG_NAMESPACE(dbg_lib, dbg_context_msg)(DBG_OUTPUT_PREFIX, __dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, LIB_DBG_NAMESPACE(dbg_lib, var_out)(__DBG_STRINGIFY_MACRO((OnTradeTransaction_runtime)), (OnTradeTransaction_runtime)), "=", DBG_MSG_FORMAT_RIGHT_COLUMN_SPACER));
            LIB_DBG_NAMESPACE(dbg_lib, dbg_trace_end)(__dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, NULL);
            return;
        };
        #define OnTradeTransaction      dbg_OnTradeTransaction
    
    #endif
    #ifdef LIB_DEBUG_TRACE_EVENT_HANDLER_ONBOOKEVENT
        void OnBookEvent(const string& symbol)
        {
            LIB_DBG_NAMESPACE(dbg_lib, dbg_printf)(LIB_DBG_NAMESPACE(dbg_lib, dbg_context_msg)(DBG_OUTPUT_PREFIX, __dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, LIB_DBG_NAMESPACE(dbg_lib, var_out)(__DBG_STRINGIFY_MACRO((symbol)),   (symbol)),  "=", DBG_MSG_FORMAT_RIGHT_COLUMN_SPACER));
            ulong call_counter = NULL;
            LIB_DBG_NAMESPACE(dbg_lib, dbg_trace_begin)(__dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, call_counter);
            const ulong start = GetMicrosecondCount();
            dbg_OnBookEvent();
            const ulong OnBookEvent_runtime = GetMicrosecondCount() - start;
            LIB_DBG_NAMESPACE(dbg_lib, dbg_printf)(LIB_DBG_NAMESPACE(dbg_lib, dbg_context_msg)(DBG_OUTPUT_PREFIX, __dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, LIB_DBG_NAMESPACE(dbg_lib, var_out)(__DBG_STRINGIFY_MACRO((OnBookEvent_runtime)), (OnBookEvent_runtime)), "=", DBG_MSG_FORMAT_RIGHT_COLUMN_SPACER));
            LIB_DBG_NAMESPACE(dbg_lib, dbg_trace_end)(__dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, NULL);
            return;
        };
        #define OnBookEvent             dbg_OnBookEvent
    
    #endif
    #ifdef LIB_DEBUG_TRACE_EVENT_HANDLER_ONCHARTEVENT
        void OnChartEvent(const int _id, const long& lparam, const double& dparam, const string& sparam)
        {
            const ENUM_CHART_EVENT id = (ENUM_CHART_EVENT)_id;
            LIB_DBG_NAMESPACE(dbg_lib, dbg_printf)(LIB_DBG_NAMESPACE(dbg_lib, dbg_context_msg)(DBG_OUTPUT_PREFIX, __dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, LIB_DBG_NAMESPACE(dbg_lib, var_out)(__DBG_STRINGIFY_MACRO((id)),       (id)),      "=", DBG_MSG_FORMAT_RIGHT_COLUMN_SPACER));
            LIB_DBG_NAMESPACE(dbg_lib, dbg_printf)(LIB_DBG_NAMESPACE(dbg_lib, dbg_context_msg)(DBG_OUTPUT_PREFIX, __dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, LIB_DBG_NAMESPACE(dbg_lib, var_out)(__DBG_STRINGIFY_MACRO((lparam)),   (lparam)),  "=", DBG_MSG_FORMAT_RIGHT_COLUMN_SPACER));
            LIB_DBG_NAMESPACE(dbg_lib, dbg_printf)(LIB_DBG_NAMESPACE(dbg_lib, dbg_context_msg)(DBG_OUTPUT_PREFIX, __dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, LIB_DBG_NAMESPACE(dbg_lib, var_out)(__DBG_STRINGIFY_MACRO((dparam)),   (dparam)),  "=", DBG_MSG_FORMAT_RIGHT_COLUMN_SPACER));
            LIB_DBG_NAMESPACE(dbg_lib, dbg_printf)(LIB_DBG_NAMESPACE(dbg_lib, dbg_context_msg)(DBG_OUTPUT_PREFIX, __dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, LIB_DBG_NAMESPACE(dbg_lib, var_out)(__DBG_STRINGIFY_MACRO((sparam)),   (sparam)),  "=", DBG_MSG_FORMAT_RIGHT_COLUMN_SPACER));
            ulong call_counter = NULL;
            LIB_DBG_NAMESPACE(dbg_lib, dbg_trace_begin)(__dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, call_counter);
            const ulong start = GetMicrosecondCount();
            dbg_OnChartEvent(_id, lparam, dparam, sparam);
            const ulong OnChartEvent_runtime = GetMicrosecondCount() - start;
            LIB_DBG_NAMESPACE(dbg_lib, dbg_printf)(LIB_DBG_NAMESPACE(dbg_lib, dbg_context_msg)(DBG_OUTPUT_PREFIX, __dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, LIB_DBG_NAMESPACE(dbg_lib, var_out)(__DBG_STRINGIFY_MACRO((OnChartEvent_runtime)), (OnChartEvent_runtime)), "=", DBG_MSG_FORMAT_RIGHT_COLUMN_SPACER));
            LIB_DBG_NAMESPACE(dbg_lib, dbg_trace_end)(__dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, NULL);
            return;
        };
        #define OnChartEvent            dbg_OnChartEvent
    
    #endif
    #ifdef LIB_DEBUG_TRACE_EVENT_HANDLER_ONTESTER
        double OnTester()
        {
            ulong call_counter = NULL;
            LIB_DBG_NAMESPACE(dbg_lib, dbg_trace_begin)(__dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, call_counter);
            const ulong start = GetMicrosecondCount();
            const double OnTester_return = dbg_OnTester();
            const ulong OnTester_runtime = GetMicrosecondCount() - start;
            LIB_DBG_NAMESPACE(dbg_lib, dbg_printf)(LIB_DBG_NAMESPACE(dbg_lib, dbg_context_msg)(DBG_OUTPUT_PREFIX, __dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, LIB_DBG_NAMESPACE(dbg_lib, var_out)(__DBG_STRINGIFY_MACRO((OnTester_runtime)), (OnTester_runtime)), "=", DBG_MSG_FORMAT_RIGHT_COLUMN_SPACER));
            LIB_DBG_NAMESPACE(dbg_lib, dbg_trace_end)(__dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, LIB_DBG_NAMESPACE(dbg_lib, var_out)(__DBG_STRINGIFY_MACRO((OnTester_return)), OnTester_return));
            return(OnTester_return);
        };
        #define OnTester                dbg_OnTester
    
    #endif
    #ifdef LIB_DEBUG_TRACE_EVENT_HANDLER_ONTESTERINIT
        int OnTesterInit()
        {
            if(LIB_DBG_NAMESPACE(dbg_lib, tester_ptr) == NULL)
            {
                LIB_DBG_NAMESPACE(dbg_lib, tester_ptr) = new LIB_DBG_NAMESPACE(dbg_lib, __event_handler_fptr)(dbg_OnTesterInit);
                LIB_DBG_NAMESPACE(dbg_lib, auto_destroy_tester_object) = LIB_DBG_NAMESPACE(dbg_lib, tester_ptr);
            }
            ulong call_counter = NULL;
            LIB_DBG_NAMESPACE(dbg_lib, dbg_trace_begin)(__dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, call_counter);
            const ulong start = GetMicrosecondCount();
            const int OnTesterInit_return = LIB_DBG_NAMESPACE(dbg_lib, tester_ptr).call_handler();
            const ulong OnTesterInit_runtime = GetMicrosecondCount() - start;
            LIB_DBG_NAMESPACE(dbg_lib, dbg_printf)(LIB_DBG_NAMESPACE(dbg_lib, dbg_context_msg)(DBG_OUTPUT_PREFIX, __dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, LIB_DBG_NAMESPACE(dbg_lib, var_out)(__DBG_STRINGIFY_MACRO((OnTesterInit_runtime)), (OnTesterInit_runtime)), "=", DBG_MSG_FORMAT_RIGHT_COLUMN_SPACER));
            LIB_DBG_NAMESPACE(dbg_lib, dbg_trace_end)(__dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, LIB_DBG_NAMESPACE(dbg_lib, var_out)(__DBG_STRINGIFY_MACRO((OnTesterInit_return)), OnTesterInit_return));
            return(OnTesterInit_return);
        };
        #define OnTesterInit            dbg_OnTesterInit
    
    #endif
    #ifdef LIB_DEBUG_TRACE_EVENT_HANDLER_ONTESTERDEINIT
        void OnTesterDeinit()
        {
            ulong call_counter = NULL;
            LIB_DBG_NAMESPACE(dbg_lib, dbg_trace_begin)(__dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, call_counter);
            const ulong start = GetMicrosecondCount();
            dbg_OnTesterDeinit();
            const ulong OnTesterDeinit_runtime = GetMicrosecondCount() - start;
            LIB_DBG_NAMESPACE(dbg_lib, dbg_printf)(LIB_DBG_NAMESPACE(dbg_lib, dbg_context_msg)(DBG_OUTPUT_PREFIX, __dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, LIB_DBG_NAMESPACE(dbg_lib, var_out)(__DBG_STRINGIFY_MACRO((OnTesterDeinit_runtime)), (OnTesterDeinit_runtime)), "=", DBG_MSG_FORMAT_RIGHT_COLUMN_SPACER));
            LIB_DBG_NAMESPACE(dbg_lib, dbg_trace_end)(__dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, NULL);
            return;
        };
        #define OnTesterDeinit          dbg_OnTesterDeinit
    
    #endif
    #ifdef LIB_DEBUG_TRACE_EVENT_HANDLER_ONTESTERPASS
        void OnTesterPass()
        {
            ulong call_counter = NULL;
            LIB_DBG_NAMESPACE(dbg_lib, dbg_trace_begin)(__dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, call_counter);
            const ulong start = GetMicrosecondCount();
            dbg_OnTesterPass();
            const ulong OnTesterPass_runtime = GetMicrosecondCount() - start;
            LIB_DBG_NAMESPACE(dbg_lib, dbg_printf)(LIB_DBG_NAMESPACE(dbg_lib, dbg_context_msg)(DBG_OUTPUT_PREFIX, __dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, LIB_DBG_NAMESPACE(dbg_lib, var_out)(__DBG_STRINGIFY_MACRO((OnTesterPass_runtime)), (OnTesterPass_runtime)), "=", DBG_MSG_FORMAT_RIGHT_COLUMN_SPACER));
            LIB_DBG_NAMESPACE(dbg_lib, dbg_trace_end)(__dbg_program_session_lifetime.program_init_name, __FUNCTION__, 0, NULL);
            return;
        };
        #define OnTesterPass            dbg_OnTesterPass
    
    #endif
    #ifdef LIB_DEBUG_TRACE_HEAP_MEMORY
        #define new                                         dbg_heap_obj_trace._location_recorder(__LINE__, __FILE__, __FUNCTION__) = new 
        #define delete                                      delete dbg_heap_obj_trace -

    #endif

#endif
//
/////////////////////////////////////////////////////////////////////////////////////////////////////


//
// END Debugging support
//*********************************************************************************************************************************************************/
#endif // LIB_DBG_DEBUG_GLOBAL_EVENT_HANDLER_TRACER_MQH_INCLUDED
